from datetime import datetime, timedelta
import secrets
from sqlalchemy import Column, Integer, String, DateTime, ForeignKey
from sqlalchemy.orm import relationship
from models.base import Base, Session

class UserSession(Base):
    __tablename__ = 'user_sessions'
    
    id = Column(Integer, primary_key=True)
    user_id = Column(Integer, ForeignKey('users.id'))
    session_token = Column(String(64), unique=True, nullable=False)
    expires_at = Column(DateTime, nullable=False)
    created_at = Column(DateTime, default=datetime.utcnow)
    ip_address = Column(String(45))
    user_agent = Column(String(255))
    
    user = relationship("User", backref="sessions")

    @classmethod
    def create_session(cls, user_id, ip_address=None, user_agent=None):
        session = Session()
        try:
            # Generate a secure random token
            token = secrets.token_hex(32)
            # Set expiration to 30 days from now
            expires_at = datetime.utcnow() + timedelta(days=30)
            
            user_session = cls(
                user_id=user_id,
                session_token=token,
                expires_at=expires_at,
                ip_address=ip_address,
                user_agent=user_agent
            )
            session.add(user_session)
            session.commit()
            return token
        finally:
            session.close()

    @classmethod
    def validate_session(cls, token):
        session = Session()
        try:
            user_session = (session.query(cls)
                          .filter(cls.session_token == token,
                                 cls.expires_at > datetime.utcnow())
                          .first())
            return user_session.user_id if user_session else None
        finally:
            session.close()

    @classmethod
    def invalidate_session(cls, token):
        session = Session()
        try:
            user_session = session.query(cls).filter(cls.session_token == token).first()
            if user_session:
                session.delete(user_session)
                session.commit()
                return True
            return False
        finally:
            session.close()